@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2005-2006 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -25,6 +25,7 @@
 
 #include <string.h>
 #include <glib.h>
+#include <stdlib.h>
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 
@@ -75,6 +76,20 @@ static guint	     signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GpmHal, gpm_hal, G_TYPE_OBJECT)
 
+/**
+ * gpm_hal_error_quark:
+ * Return value: Our personal error quark.
+ **/
+GQuark
+gpm_hal_error_quark (void)
+{
+	static GQuark quark = 0;
+	if (!quark) {
+		quark = g_quark_from_static_string ("gpm_hal_error");
+	}
+	return quark;
+}
+
 /**
  * gpm_hal_is_running:
  *
@@ -91,82 +106,11 @@ gpm_hal_is_running (GpmHal *hal)
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
 
-	running = gpm_hal_device_get_string (hal, HAL_ROOT_COMPUTER, "info.udi", &udi);
+	running = gpm_hal_device_get_string (hal, HAL_ROOT_COMPUTER, "info.udi", &udi, NULL);
 	g_free (udi);
 	return running;
 }
 
-/**
- * gpm_hal_device_rescan:
- *
- * @hal: This class instance
- * @udi: The HAL UDI
- * Return value: Success, true if we rescan'd
- *
- * Rescans a HAL device manually
- **/
-static gboolean
-gpm_hal_device_rescan (GpmHal *hal, const gchar *udi)
-{
-	gint retval = 0;
-	DBusGProxy *proxy;
-	GError *error = NULL;
-	gboolean ret;
-
-	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
-	g_return_val_if_fail (udi != NULL, FALSE);
-
-	gpm_debug ("Doing Rescan (%s)", udi);
-	proxy = dbus_g_proxy_new_for_name (hal->priv->connection,
-					   HAL_DBUS_SERVICE,
-					   udi,
-					   HAL_DBUS_INTERFACE_DEVICE);
-	ret = dbus_g_proxy_call (proxy, "Rescan", &error,
-				 G_TYPE_INVALID,
-				 G_TYPE_BOOLEAN, &retval, G_TYPE_INVALID);
-	if (error) {
-		gpm_debug ("ERROR: %s", error->message);
-		g_error_free (error);
-	}
-	if (ret == FALSE || retval != 0) {
-		/* abort as the DBUS method failed */
-		gpm_warning ("Rescan failed!");
-		return FALSE;
-	}
-	g_object_unref (G_OBJECT (proxy));
-	return TRUE;
-}
-
-/**
- * gpm_hal_device_rescan_capability:
- *
- * @hal: This class instance
- * @capability: The HAL capability, e.g. laptop_panel
- * Return value: Success, true if we found and devices
- *
- * Rescans all devices of a specific capability manually
- **/
-gboolean
-gpm_hal_device_rescan_capability (GpmHal *hal, const gchar *capability)
-{
-	gchar **devices;
-	guint i;
-
-	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
-	g_return_val_if_fail (capability != NULL, FALSE);
-
-	gpm_hal_device_find_capability (hal, capability, &devices);
-	if (devices == NULL) {
-		gpm_debug ("No devices of capability %s", capability);
-		return FALSE;
-	}
-	for (i = 0; devices[i]; i++) {
-		gpm_hal_device_rescan (hal, devices[i]);
-	}
-	gpm_hal_free_capability (hal, devices);
-	return TRUE;
-}
-
 /**
  * gpm_hal_device_get_bool:
  *
@@ -180,10 +124,10 @@ gboolean
 gpm_hal_device_get_bool (GpmHal      *hal,
 			 const gchar *udi,
 			 const gchar *key,
-			 gboolean    *value)
+			 gboolean    *value,
+			 GError     **error)
 {
 	DBusGProxy *proxy;
-	GError *error = NULL;
 	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
@@ -195,23 +139,20 @@ gpm_hal_device_get_bool (GpmHal      *hal,
 					       HAL_DBUS_SERVICE,
 					       udi,
 					       HAL_DBUS_INTERFACE_DEVICE);
-	ret = dbus_g_proxy_call (proxy, "GetPropertyBoolean", &error,
+	if (proxy == NULL) {
+		gpm_warning ("GetPropertyBoolean failed to get proxy for %s", udi);
+		return FALSE;
+	}
+	ret = dbus_g_proxy_call (proxy, "GetPropertyBoolean", error,
 				 G_TYPE_STRING, key,
 				 G_TYPE_INVALID,
 				 G_TYPE_BOOLEAN, value,
 				 G_TYPE_INVALID);
-	if (error) {
-		gpm_debug ("ERROR: %s", error->message);
-		g_error_free (error);
-	}
 	if (ret == FALSE) {
-		/* abort as the DBUS method failed */
 		*value = FALSE;
-		gpm_debug ("GetPropertyBoolean failed!");
-		return FALSE;
 	}
 	g_object_unref (G_OBJECT (proxy));
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -229,10 +170,10 @@ gboolean
 gpm_hal_device_get_string (GpmHal      *hal,
 			   const gchar *udi,
 			   const gchar *key,
-			   gchar      **value)
+			   gchar      **value,
+			   GError     **error)
 {
 	DBusGProxy *proxy = NULL;
-	GError *error = NULL;
 	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
@@ -244,23 +185,20 @@ gpm_hal_device_get_string (GpmHal      *hal,
 					       HAL_DBUS_SERVICE,
 					       udi,
 					       HAL_DBUS_INTERFACE_DEVICE);
-	ret = dbus_g_proxy_call (proxy, "GetPropertyString", &error,
+	if (proxy == NULL) {
+		gpm_warning ("GetPropertyString failed to get proxy for %s", udi);
+		return FALSE;
+	}
+	ret = dbus_g_proxy_call (proxy, "GetPropertyString", error,
 				 G_TYPE_STRING, key,
 				 G_TYPE_INVALID,
 				 G_TYPE_STRING, value,
 				 G_TYPE_INVALID);
-	if (error) {
-		gpm_debug ("ERROR: %s", error->message);
-		g_error_free (error);
-	}
 	if (ret == FALSE) {
-		/* abort as the DBUS method failed */
 		*value = NULL;
-		gpm_debug ("GetPropertyString failed!");
-		return FALSE;
 	}
 	g_object_unref (G_OBJECT (proxy));
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -276,10 +214,10 @@ gboolean
 gpm_hal_device_get_int (GpmHal      *hal,
 			const gchar *udi,
 			const gchar *key,
-			gint        *value)
+			gint        *value,
+			GError     **error)
 {
 	DBusGProxy *proxy;
-	GError *error = NULL;
 	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
@@ -291,23 +229,20 @@ gpm_hal_device_get_int (GpmHal      *hal,
 					       HAL_DBUS_SERVICE,
 					       udi,
 					       HAL_DBUS_INTERFACE_DEVICE);
-	ret = dbus_g_proxy_call (proxy, "GetPropertyInteger", &error,
+	if (proxy == NULL) {
+		gpm_warning ("GetPropertyInteger failed to get proxy for %s", udi);
+		return FALSE;
+	}
+	ret = dbus_g_proxy_call (proxy, "GetPropertyInteger", error,
 				 G_TYPE_STRING, key,
 				 G_TYPE_INVALID,
 				 G_TYPE_INT, value,
 				 G_TYPE_INVALID);
-	if (error) {
-		gpm_debug ("ERROR: %s", error->message);
-		g_error_free (error);
-	}
 	if (ret == FALSE) {
-		/* abort as the DBUS method failed */
 		*value = 0;
-		gpm_debug ("GetPropertyInteger failed!");
-		return FALSE;
 	}
 	g_object_unref (G_OBJECT (proxy));
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -319,13 +254,14 @@ gboolean
 gpm_hal_device_get_uint (GpmHal      *hal,
 			 const gchar *udi,
 			 const gchar *key,
-			 guint       *value)
+			 guint       *value,
+			 GError     **error)
 {
 	gint tvalue;
 	gboolean ret;
 
 	/* bodge */
-	ret = gpm_hal_device_get_int (hal, udi, key, &tvalue);
+	ret = gpm_hal_device_get_int (hal, udi, key, &tvalue, error);
 	*value = (guint) tvalue;
 	return ret;
 }
@@ -341,10 +277,10 @@ gpm_hal_device_get_uint (GpmHal      *hal,
 gboolean
 gpm_hal_device_find_capability (GpmHal      *hal,
 				const gchar *capability,
-				gchar     ***value)
+				gchar     ***value,
+				GError     **error)
 {
 	DBusGProxy *proxy = NULL;
-	GError *error = NULL;
 	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
@@ -355,23 +291,20 @@ gpm_hal_device_find_capability (GpmHal      *hal,
 					       HAL_DBUS_SERVICE,
 					       HAL_DBUS_PATH_MANAGER,
 					       HAL_DBUS_INTERFACE_MANAGER);
-	ret = dbus_g_proxy_call (proxy, "FindDeviceByCapability", &error,
+	if (proxy == NULL) {
+		gpm_warning ("FindDeviceByCapability failed to get proxy");
+		return FALSE;
+	}
+	ret = dbus_g_proxy_call (proxy, "FindDeviceByCapability", error,
 				 G_TYPE_STRING, capability,
 				 G_TYPE_INVALID,
 				 G_TYPE_STRV, value,
 				 G_TYPE_INVALID);
-	if (error) {
-		gpm_debug ("ERROR: %s", error->message);
-		g_error_free (error);
-	}
 	if (ret == FALSE) {
-		/* abort as the DBUS method failed */
 		*value = NULL;
-		gpm_warning ("FindDeviceByCapability failed!");
-		return FALSE;
 	}
 	g_object_unref (G_OBJECT (proxy));
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -386,10 +319,10 @@ gboolean
 gpm_hal_device_has_capability (GpmHal      *hal,
 			       const gchar *udi,
 			       const gchar *capability,
-			       gboolean    *has_capability)
+			       gboolean    *has_capability,
+			       GError     **error)
 {
 	DBusGProxy *proxy = NULL;
-	GError *error = NULL;
 	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
@@ -401,23 +334,20 @@ gpm_hal_device_has_capability (GpmHal      *hal,
 					   HAL_DBUS_SERVICE,
 					   udi,
 					   HAL_DBUS_INTERFACE_DEVICE);
-	ret = dbus_g_proxy_call (proxy, "QueryCapability", &error,
+	if (proxy == NULL) {
+		gpm_warning ("QueryCapability failed to get proxy for %s", udi);
+		return FALSE;
+	}
+	ret = dbus_g_proxy_call (proxy, "QueryCapability", error,
 				 G_TYPE_STRING, udi,
 				 G_TYPE_INVALID,
 				 G_TYPE_BOOLEAN, has_capability,
 				 G_TYPE_INVALID);
-	if (error) {
-		gpm_debug ("ERROR: %s", error->message);
-		g_error_free (error);
-	}
 	if (ret == FALSE) {
-		/* abort as the DBUS method failed */
 		*has_capability = FALSE;
-		gpm_warning ("QueryCapability failed!");
-		return FALSE;
 	}
 	g_object_unref (G_OBJECT (proxy));
-	return TRUE;
+	return ret;
 }
 
 /**
@@ -426,7 +356,7 @@ gpm_hal_device_has_capability (GpmHal      *hal,
  * @hal: This class instance
  * @value: The list of strings to free
  *
- * Frees value result of gpm_hal_device_find_capability
+ * Frees value result of gpm_hal_device_find_capability. Safe to call with NULL.
  **/
 void
 gpm_hal_free_capability (GpmHal *hal, gchar **value)
@@ -434,8 +364,10 @@ gpm_hal_free_capability (GpmHal *hal, gchar **value)
 	gint i;
 
 	g_return_if_fail (GPM_IS_HAL (hal));
-	g_return_if_fail (value != NULL);
 
+	if (value == NULL) {
+		return;
+	}
 	for (i=0; value[i]; i++) {
 		g_free (value[i]);
 	}
@@ -456,12 +388,13 @@ gpm_hal_num_devices_of_capability (GpmHal *hal, const gchar *capability)
 {
 	gint i;
 	gchar **names;
+	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), 0);
 	g_return_val_if_fail (capability != NULL, 0);
 
-	gpm_hal_device_find_capability (hal, capability, &names);
-	if (names == NULL) {
+	ret = gpm_hal_device_find_capability (hal, capability, &names, NULL);
+	if (ret == FALSE) {
 		gpm_debug ("No devices of capability %s", capability);
 		return 0;
 	}
@@ -492,20 +425,21 @@ gpm_hal_num_devices_of_capability_with_value (GpmHal      *hal,
 	gint i;
 	gint valid = 0;
 	gchar **names;
+	gboolean ret;
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), 0);
 	g_return_val_if_fail (capability != NULL, 0);
 	g_return_val_if_fail (key != NULL, 0);
 	g_return_val_if_fail (value != NULL, 0);
 
-	gpm_hal_device_find_capability (hal, capability, &names);
-	if (names == NULL) {
+	ret = gpm_hal_device_find_capability (hal, capability, &names, NULL);
+	if (ret == FALSE) {
 		gpm_debug ("No devices of capability %s", capability);
 		return 0;
 	}
 	for (i = 0; names[i]; i++) {
 		gchar *type = NULL;
-		gpm_hal_device_get_string (hal, names[i], key, &type);
+		gpm_hal_device_get_string (hal, names[i], key, &type, NULL);
 		if (type != NULL) {
 			if (strcmp (type, value) == 0)
 				valid++;
@@ -1112,7 +1046,8 @@ gpm_hal_init (GpmHal *hal)
 				  HAL_DBUS_PATH_MANAGER,
 				  HAL_DBUS_INTERFACE_MANAGER);
 	if (proxy == NULL) {
-		gpm_critical_error ("Either HAL or DBUS are not working!");
+		gpm_warning ("Either HAL or DBUS are not working!\n");
+		exit (0);
 	}
 
 	/* get the power connection */
@@ -1123,7 +1058,8 @@ gpm_hal_init (GpmHal *hal)
 				  HAL_ROOT_COMPUTER,
 				  HAL_DBUS_INTERFACE_POWER);
 	if (proxy == NULL) {
-		gpm_critical_error ("HAL does not support power management!");
+		gpm_warning ("HAL does not support power management, exiting!\n");
+		exit (0);
 	}
 
 	g_signal_connect (hal->priv->gproxy_manager, "proxy-status",
@@ -1183,7 +1119,7 @@ gpm_hal_is_laptop (GpmHal *hal)
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
 
 	/* always present */
-	gpm_hal_device_get_string (hal, HAL_ROOT_COMPUTER, "system.formfactor", &formfactor);
+	gpm_hal_device_get_string (hal, HAL_ROOT_COMPUTER, "system.formfactor", &formfactor, NULL);
 	if (formfactor == NULL) {
 		gpm_debug ("system.formfactor not set!");
 		/* no need to free */
@@ -1213,7 +1149,7 @@ gpm_hal_has_power_management (GpmHal *hal)
 
 	g_return_val_if_fail (GPM_IS_HAL (hal), FALSE);
 
-	gpm_hal_device_get_string (hal, HAL_ROOT_COMPUTER, "power_management.type", &ptype);
+	gpm_hal_device_get_string (hal, HAL_ROOT_COMPUTER, "power_management.type", &ptype, NULL);
 	/* this key only has to exist to be pm okay */
 	if (ptype) {
 		gpm_debug ("Power management type : %s", ptype);
@@ -1242,7 +1178,7 @@ gpm_hal_can_suspend (GpmHal *hal)
 	/* TODO: Change to can_suspend when rely on newer HAL */
 	exists = gpm_hal_device_get_bool (hal, HAL_ROOT_COMPUTER,
 					  "power_management.can_suspend_to_ram",
-					  &can_suspend);
+					  &can_suspend, NULL);
 	if (exists == FALSE) {
 		gpm_warning ("gpm_hal_can_suspend: Key can_suspend_to_ram missing");
 		return FALSE;
@@ -1269,7 +1205,7 @@ gpm_hal_can_hibernate (GpmHal *hal)
 	/* TODO: Change to can_hibernate when rely on newer HAL */
 	exists = gpm_hal_device_get_bool (hal, HAL_ROOT_COMPUTER,
 					  "power_management.can_suspend_to_disk",
-					  &can_hibernate);
+					  &can_hibernate, NULL);
 	if (exists == FALSE) {
 		gpm_warning ("gpm_hal_can_hibernate: Key can_suspend_to_disk missing");
 		return FALSE;
@@ -1540,10 +1476,9 @@ gpm_hal_has_hibernate_error (GpmHal *hal, gboolean *state)
  * Tells HAL to try and clear the suspend error as we appear to be okay
  **/
 gboolean
-gpm_hal_clear_suspend_error (GpmHal *hal)
+gpm_hal_clear_suspend_error (GpmHal *hal, GError **error)
 {
-#if HAVE_HAL_NEW
-	GError *error = NULL;
+#if 0
 	gboolean ret;
 	DBusGProxy *proxy;
 
@@ -1553,21 +1488,16 @@ gpm_hal_clear_suspend_error (GpmHal *hal)
 	proxy = gpm_proxy_get_proxy (hal->priv->gproxy_power);
 	if (proxy == NULL) {
 		gpm_warning ("not connected");
+		*error = g_error_new (gpm_hal_error_quark (),
+				      GPM_HAL_NOT_CONNECTED,
+				      "Not connected");
 		return FALSE;
 	}
 
 	gpm_debug ("Doing SuspendClearError");
-	ret = dbus_g_proxy_call (proxy, "SuspendClearError", &error,
+	ret = dbus_g_proxy_call (proxy, "SuspendClearError", error,
 				 G_TYPE_INVALID, G_TYPE_INVALID);
-	if (error) {
-		gpm_debug ("ERROR: %s", error->message);
-		g_error_free (error);
-	}
-	if (ret == FALSE) {
-		/* abort as the DBUS method failed */
-		gpm_warning ("SuspendClearError failed!");
-		return FALSE;
-	}
+	return ret;
 #endif
 	return TRUE;
 }
@@ -1581,10 +1511,9 @@ gpm_hal_clear_suspend_error (GpmHal *hal)
  * Tells HAL to try and clear the hibernate error as we appear to be okay
  **/
 gboolean
-gpm_hal_clear_hibernate_error (GpmHal *hal)
+gpm_hal_clear_hibernate_error (GpmHal *hal, GError **error)
 {
-#if HAVE_HAL_NEW
-	GError *error = NULL;
+#if 0
 	gboolean ret;
 	DBusGProxy *proxy;
 
@@ -1598,17 +1527,9 @@ gpm_hal_clear_hibernate_error (GpmHal *hal)
 	}
 
 	gpm_debug ("Doing HibernateClearError");
-	ret = dbus_g_proxy_call (proxy, "HibernateClearError", &error,
+	ret = dbus_g_proxy_call (proxy, "HibernateClearError", error,
 				 G_TYPE_INVALID, G_TYPE_INVALID);
-	if (error) {
-		gpm_debug ("ERROR: %s", error->message);
-		g_error_free (error);
-	}
-	if (ret == FALSE) {
-		/* abort as the DBUS method failed */
-		gpm_warning ("HibernateClearError failed!");
-		return FALSE;
-	}
+	return ret;
 #endif
 	return TRUE;
 }
